@@ -104,9 +104,10 @@ setMethod(".gtree",
 
             
             ## call in autoscroll if requested -- has issues with sizing
-            if(getWithDefault(theArgs$do.autoscroll, FALSE)) {
-              tcl("autoscroll", xscr)
-              tcl("autoscroll", yscr)
+            if(getWithDefault(theArgs$do.autoscroll, TRUE) &&
+               windowingsystem() != "aqua") {
+              tcl("autoscroll::autoscroll", xscr)
+              tcl("autoscroll::autoscroll", yscr)
             }
 
             ## turn on alternating shading if more than 1 column
@@ -350,6 +351,7 @@ setMethod(".update",
             ## add children
             .treeAddOffspring(tr, parent="", os, whichHaveOffspring,
                               icons=icons)
+            invisible()
           })
 
 ## index returns the indices
@@ -482,7 +484,7 @@ setReplaceMethod(".size",
                    
                    ## set width
                    sapply(2:n, function(j) {
-                     tcl(tr, "column", j - 2, "-width",widths[j])
+                     tcl(tr, "column", j - 2, width=widths[j], stretch=TRUE, anchor="w")
                    })
                    tcl(tr,"column","#0","-width", widths[1])
                    
